function SRSttStp(action)
%SRSttStp: start and stop shortrec recording
%
% Note0: This function is one of the few which
% use the shortrec variable sr as a global variable.
% Using sr as a global keeps the loop running quickly
% and greatly eases the programmer's burden.
%
% Note1: This function executes a loop which exists on one of
% several conditions:
%       a) The user clicks on the stop button.
%       b) A finite number of rounds of recording is deisred,
%          and that number of stimulations has been applied.
%
% Note2: the while loop determines whether or not the recording
% is "on" by querying the 'String' property of the start/stop button
% (whose 'Tag' property is 'SRstartstop').
% Pressing the 'start' button sets the 'String' property to 'stop',
% and vice versa.

% Note3: the while loop determines whether or not the # of recording
% counts is finite or infinite by querying the 'Value' property of
% the popup menu, whose 'Tag' is 'NumRecs'.  A 'Value' of 1 is indicates
% a finite number of stimulations is desired, while a 'Value' of 2
% indicates that an infinite number of stimulations is desired.

global sr
global sr_old
global tp
global ec

% These local variables are used to store data temporarily
% while the routine checks for input from the GUI.  If the
% user presses "stop" and wants to keep the data from the
% last round of recording, the data in these variables is
% thrown out.
%cm_data = zeros(sr.cm_num_samples,2);
%nr_data = zeros(8,sr.nr_num_samples);

switch action

  case 'start'

    % Replace 'start' button with 'stop' button
    set(findobj('Tag','SRstartstop'),'BackgroundColor',[1 0 0], ...
    'String','stop','Callback','SRSttStp stop')

    % Store current date and time
    sr.datestr = datestr(now,0);

    % Read current Monitor Offsets if stimulating electrically
    if sr.stim_type == 2
      sr.cm_offsets = SRDapRMO;
    else
      sr.cm_offsets = [0 0];
    end

    % Read eight channel amplifier settings and update screen
    ECDapStt
    ec.channel_settings = ECDapRd(zeros(1,8));
    for i = 1:8
      set(findobj('Tag',[char(64+i) 'Setting']),'String', ...
        [char(64+i) num2str(ec.channel_settings(i))]);
    end

    % Close channel-setting-reading routine.
    ECDapStp

    % Start appropriate routines before entering while loop.
    % These are determined by the stim_type.
    switch sr.stim_type
      % Electric stimulation
      case 2
        DapSndLn('start twplsb,twplsa,shrtreca,shrtrecb');
      % Optic Stimulation
     case 3
        DapSndLn('start ledstima,ledstimb');
    end

    % Cause a short delay to make sure DAP is configured
    % before any other actions are taken.
    tic;
    while (toc < .1)
    end

    % Set the number of rounds of recording
    NumLeft = sr.num_recs;
    finite = (get(findobj('Tag','NumRecs'),'Value') == 1);

    while (NumLeft ~= 0) | (~finite)

      % Cause a short delay.  Increasing the delay will decrease
      % the frequency of recording and vice-versa.
      tic;
      while (toc < .5)
      end

      % draw data to screen and check for events from GUI
      drawnow;

      % Break out of loop if user pressed stop button.
      if strcmp(get(findobj('Tag','SRstartstop'),'String'),'start')
        break
      end

      % Update finite/infinite property in case user pressed button.
      finite = (get(findobj('Tag','NumRecs'),'Value') == 1);

      % Trigger the DAP board to record data.  For electric
      % stimulation, this is done by sending the stimulation
      % pulse.  For none/optic stimulation, this is done by
      % starting the recording routines.
      switch sr.stim_type
        case {1,3}
          DapSndLn('start shrtrecc,shrtrecd');
        case 2
          TPDapSnd(reshape([TPTrgOut(tp)'; TPPlsOut(tp)'],tp.num_samples*2,1));
      end

      % Keep old data
      sr_old = sr;

      % Read data from DAP and plot.  If stim_type is not 2 (electric),
      % the cm_data field will just be filled with zeros.
      [sr.cm_data,sr.nr_data] = SRDapRd(sr.stim_type);

      % We need to stop the recording "processing" procedure
      % shrtrecd if it was started above.  shrtrecc stops by
      % itself when it has collected the requisite number of
      % samples, and thus doesn't need to be explicitly stopped.
      if ismember(sr.stim_type,[1 3])
        DapSndLn('stop shrtrecd');
      end

      % Transfer data from local variables to sr variable
     % sr.cm_data = cm_data;
     % sr.nr_data = nr_data;

      % Plot Data
      SRPltDat(sr,tp);

     % Decrement counter and update screen if a finite # rounds
      if finite
         NumLeft = abs(NumLeft) - 1;
         Str = sprintf('%d',NumLeft);
         set(findobj('Tag','count','Style','text'),'String',Str)
      end

      % Force changes to screen.
%      drawnow;

    end

    % Stop dap routines and redraw start button only
    % if stop button was not pressed by user up until now.
    if ~strcmp(get(findobj('Tag','SRstartstop'),'String'),'start')
      SRSttStp('stop');
    end

  case 'stop'
      % Stop dap routines
      switch sr.stim_type
        case 2
          DapSndLn('stop twplsa,twplsb,shrtreca,shrtrecb');
        case 3
          DapSndLn('stop ledstima,ledstimb');
          % zero the output in case the led is still on
          DapSndLn('start ledzero');
          tic; while (toc < .1); end;
          DapSndLn('stop ledzero');
      end

      if get(findobj('Tag','NumRecs'),'Value') == 1
        Str = sprintf('%d',sr.num_recs);
        set(findobj(gcf,'Tag','count','Style','text'),'String',Str)
      end
      set(findobj('Tag','SRstartstop'),'BackgroundColor',[0 1 0], ...
      'String','start','Callback','SRSttStp start')

end






















