function [newtp,err] = TPLckWds(oldtp,action)
%TPLckWds: perform operations on widths of a shape-locked waveform

newtp = oldtp;
err = 0;

% Determine which of the three widths is the anchor, and
% assign anchor width and ratios accordingly.

if strcmp(action(1:2),'lk')
  AnchorStr = action(3:4);
else
  AnchorStr = action(1:2);
end

switch AnchorStr

  case 'w1'
    AnchorWidth = newtp.pulse1_width;

    % Ratio of pulse2_delay to AnchorWidth
    Ratio21 = newtp.d2w1_ratio;

    % Ratio of pulse2_width to AnchorWidth
    Ratio31 = 1/newtp.w1w2_ratio;

  case 'w2'
    AnchorWidth = newtp.pulse2_width;  

    % Ratio of pulse1_width to AnchorWidth
    Ratio21 = newtp.w1w2_ratio;

    % Ratio of pulse2_delay to AnchorWidth
    Ratio31 = Ratio21 * newtp.d2w1_ratio;

  case 'd2'
    AnchorWidth = newtp.pulse2_delay;

    % Ratio of pulse1_width to AnchorWidth
    Ratio21 = 1/newtp.d2w1_ratio;

    % Ratio of pulse2_width to AnchorWidth
    Ratio31 = Ratio21*(1/newtp.w1w2_ratio);

end

% Make a vector of all possible widths for the anchor width.
CandidateWidths = 0:(newtp.num_samples - 1);

% Err1(Err2) is the deviation of the "second width" ("third width")
% from the closest integer, calulated from the ratio
% secondwidth/anchorwidth (thirdwidth/anchorwidth) and the candidate
% anchor widths.
Err1 = abs( round(Ratio21 * CandidateWidths) - ...
                 (Ratio21 * CandidateWidths) );
Err2 = abs( round(Ratio31 * CandidateWidths) - ...
                 (Ratio31 * CandidateWidths) );

% Find candidate widths which have acceptable errors
OkWidths = -1 + intersect(find(Err1 < newtp.width_tolerance), ...
                          find(Err2 < newtp.width_tolerance));

% If no good widths were found, generate and error message
% and return the old waveform.
if isempty(OkWidths)
  err = 1;
  newtp = oldtp;
  h = errordlg(['Unable to find an acceptable combination of pulse ' ...
               'widths. ' ...
               'You might try decreasing the width of the waveform ' ...
               'or increasing width_tolerance.'],'TPPreset:','modal');
  figure(h)
  uiwait(h)
  return
end

% Find usable width which is closest to AnchorWidth
[dummy,ClosestWidth] = min(abs(AnchorWidth-OkWidths));


% Set new width based on instructions in action string
if strcmp(action(1:2),'lk')
  NewAnchorWidth = OkWidths(ClosestWidth);
else
  switch action

    case {'w1g','d2g','w2g'}
      if (ClosestWidth+1) <= size(OkWidths,2)
        NewAnchorWidth = OkWidths(ClosestWidth+1);
      else
        NewAnchorWidth = AnchorWidth;
      end

    case {'w1gg','d2gg','w2gg'}
      if (ClosestWidth+10) <= size(OkWidths,2)
        NewAnchorWidth = OkWidths(ClosestWidth+10);
      else
        NewAnchorWidth = AnchorWidth;
      end

    case {'w1ggg','d2ggg','w2ggg'}
      if (ClosestWidth+10) <= size(OkWidths,2)
        NewAnchorWidth = OkWidths(ClosestWidth+10);
      else
        NewAnchorWidth = AnchorWidth;
      end

    case {'w1l','d2l','w2l'}
      if (ClosestWidth-1) > 0
        NewAnchorWidth = OkWidths(ClosestWidth-1);
      else
        NewAnchorWidth = AnchorWidth;
      end

    case {'w1ll','d2ll','w2ll'}
      if (ClosestWidth-10) > 0
        NewAnchorWidth = OkWidths(ClosestWidth-10);
      else
        NewAnchorWidth = AnchorWidth;
      end

    case {'w1lll','d2lll','w2lll'}
      if (ClosestWidth-100) > 0
        NewAnchorWidth = OkWidths(ClosestWidth-100);
      else
        NewAnchorWidth = AnchorWidth;
      end

  end

end


% Reassign pulse widths and delays

switch AnchorStr

  case 'w1'
    newtp.pulse1_width = NewAnchorWidth;
    newtp.pulse2_delay = round(NewAnchorWidth * Ratio21);
    newtp.pulse2_width = round(NewAnchorWidth * Ratio31);

  case 'w2'
    newtp.pulse2_width = NewAnchorWidth;  
    newtp.pulse1_width = round(NewAnchorWidth * Ratio21);
    newtp.pulse2_delay = round(NewAnchorWidth * Ratio31);

  case 'd2'
    newtp.pulse2_delay = NewAnchorWidth;
    newtp.pulse1_width = round(NewAnchorWidth * Ratio21);
    newtp.pusle2_width = round(NewAnchorWidth * Ratio31);

end

% Check to make sure that new pulse with is within bounds

if (newtp.pulse1_delay + newtp.pulse1_width + ...
    newtp.pulse2_delay + newtp.pulse2_width) >= newtp.num_samples
  newtp = oldtp;
  if strcmp(action(1:2),'lk')
    err = 1;
    h = errordlg(['Unable to find an acceptable combination of pulse ' ...
                  'widths. ' ...
                  'You might try decreasing the width of the waveform ' ...
                  'or increasing width_tolerance.'],'TPPreset:','modal');
    figure(h)
    uiwait(h)
    return
  end
end







