/*
 *  DAPIO.C - Device driver interface routines.
 *
 *  Copyright (c) 1997, Microstar Laboratories, Inc.  
 *  All rights reserved.
 */

#include <sys/types.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include "time.h"
#include "dapio.h"

#define BUF_SIZE 2048


/* Name:    DAPGetAvail
 * Purpose: Provide convenient interface to ioctl.
 * Returns: Number of bytes available from ACCEL handle.
 */
int DAPGetAvail(int fd)
{
  int iErr, iNum;

  iErr = ioctl (fd, GETAVAIL, &iNum);
  if (iErr)
    return (-1);
  else
    return (iNum);
}


/* Name:    DAPPutAvail
 * Purpose: Provide convenient interface to ioctl.
 * Returns: Number of bytes of free space to put data into a com pipe
 */
int DAPPutAvail(int fd)
{
  int iErr, iNum;

  iErr = ioctl (fd, PUTAVAIL, &iNum);
  if (iErr)
    return (-1);
  else
    return (iNum);
}


/* Name:    DAPSendFile
 * Purpose: Convenient function to send a file of DAPL commands.
 * Returns: Number of bytes sent
 */
int DAPSendFile(int fd, char *sDAPLName)
{
  int fdDAPL, bytes, totalbytes,i;
  char sDAPLLine[1024];

  if ( (fdDAPL=open(sDAPLName, O_RDONLY)) <= 0)
    {close(fdDAPL); return(fdDAPL);}

  totalbytes = 0;

  while (1)
  {
    bytes = read(fdDAPL, sDAPLLine, 1024);
    if (bytes < 0)
      {close(fdDAPL); return(bytes);}

    if (bytes == 0)
      break;

    /* Convert \n line feed characters to \r carriage return characters */
    for (i=0;i<bytes;i++)
      if (sDAPLLine[i] == '\n')
        sDAPLLine[i] = '\r';

    bytes = write(fd, sDAPLLine, bytes);
    if (bytes < 0)
      {close(fdDAPL); return(bytes);}

    totalbytes += bytes;
  }

  close(fdDAPL);
  return (totalbytes);
}


/*  Name:    DAPSendLine
 *  Purpose: Convenient function to send one DAPL command line
 *           followed by a carriage return.
 *  Returns: Number of bytes sent
 */
int DAPSendLine(int fd, char *sDAPLLine)
{
  int bytes;

  bytes = write(fd, sDAPLLine, strlen(sDAPLLine));
  if (bytes < 0)
    return(bytes);
  return(bytes);
}


/*  Name:    DAPFlush
 *  Purpose: Removes as many bytes as possible from the communication pipe
 *  Returns: Number of bytes removed
 */
int DAPFlush(int fd)
{
  char bBuf[BUF_SIZE];
  int avail, totBytes, bytes;

  totBytes = 0;

  do
  {
    while ( (avail=DAPGetAvail(fd)) > 0)
    {
      if (avail > BUF_SIZE)
        avail = BUF_SIZE;

      bytes = read(fd,bBuf,avail);
      totBytes += bytes;
    }
    /* set delay time to 100 ms */
    usleep(100000);

  } while(DAPGetAvail(fd) > 0);

  return (totBytes);
}

 
/* Name:    DAPAccelVersion
 * Purpose: Provide convenient interface to ioctl.
 * Returns: driver version number
 */
int DAPAccelVersion(int fd)
{
  int iErr, iNum;

  iErr = ioctl (fd, VERSION, &iNum);
  if (iErr)
    return (-1);
  else
    return (iNum);
}


/* Name:    DAPSetTimeout
 * Purpose: Provide convenient interface to ioctl.
 * Returns: set the timeout value used by the read and write routines
 */
int DAPSetTimeout(int fd, unsigned long iNum)
{
  int iErr;

  iErr = ioctl (fd, SETTIMEOUT, &iNum);
  if (iErr)
    return (-1);
  else
    return (0);
}
