function TPPreset()
%TPPreset: Force twopulse waveform to closest pre-set shape

% The goal here will be to get the preset shape having the
% first pulse as near as possible to the current first pulse.

global TXT_UPDATE_FLAGS
global PULSE1_DELAY
global PULSE2_DELAY
global PULSE1_HEIGHT
global PULSE2_HEIGHT
global PULSE1_WIDTH
global PULSE2_WIDTH
global OUT_NUM_SAMPLES
global H1H2_RATIO
global D2W1_RATIO
global WIDTH_TOLERANCE
global HEIGHT_TOLERANCE

% Width ratios must be correct to within WIDTH_TOLERANCE.
% In the time domain, this translates to a maximum deviation
% in the shorter pulse of WIDTH_TOLERANCE*OUT_UPDATE_INTERVAL
% microseconds from the exact fraction of the longer pulse
% designated by H1H2_RATIO (since pulses are rectangular and
% charged-balanced, the height ratio specifies the width ratio).

% make a local copy of globals, for failure recovery.
d1 = PULSE1_DELAY;
d2 = PULSE2_DELAY;
h1 = PULSE1_HEIGHT;
h2 = PULSE2_HEIGHT;
w1 = PULSE1_WIDTH;
w2 = PULSE2_WIDTH;

% FailureFlag is set to 1 is a failure setting the pulse widths,
% and is set to 2 if there is a failure setting pulse heights.
FailureFlag = 0;

% Set pulse widths and delays.  The strategy for this optimization
% is to "try everything for pulse 1 width" and pick the best match.

Remainders1 = abs(round(abs(H1H2_RATIO)*(1:OUT_NUM_SAMPLES - 1)) - ...
                        abs(H1H2_RATIO)*(1:OUT_NUM_SAMPLES - 1));
AcceptableWidths1 = find(Remainders1 <= WIDTH_TOLERANCE);
Remainders2 = abs(round((D2W1_RATIO)*(1:OUT_NUM_SAMPLES - 1)) - ...
                        (D2W1_RATIO)*(1:OUT_NUM_SAMPLES - 1));
AcceptableWidths2 = find(Remainders2 <= WIDTH_TOLERANCE);
AcceptableWidths = intersect(AcceptableWidths1,AcceptableWidths2);

if AcceptableWidths

% This optimization function carries a penalty for
% increasing the error in the width 2, increasing the
% error in delay 2, or changing the current width 1.  
  NormDist = abs(PULSE1_WIDTH - AcceptableWidths)/OUT_NUM_SAMPLES;
  NormErr1 = Remainders1(AcceptableWidths)/WIDTH_TOLERANCE;
  NormErr2 = Remainders2(AcceptableWidths)/WIDTH_TOLERANCE;
  PenaltyFunction = ( 1 + ( .5 * ( (NormDist/.1) .^ 2 ) ) ) .* ...
                    ( 1 + NormErr1 ) .* ( 1 + NormErr2 );
  [value,index] = min(PenaltyFunction);
  PULSE1_WIDTH = AcceptableWidths(index);
  PULSE2_WIDTH = abs(H1H2_RATIO) * PULSE1_WIDTH;
  PULSE2_DELAY = D2W1_RATIO * PULSE1_WIDTH;
else
  FailureFlag = 1;
end


% Set pulse heights.  The strategy for this optimization
% is to "try everything for pulse 1 height" and pick the best match.

HeightsVector = -32768:16:32752;
Remainders = abs(round(abs(1/H1H2_RATIO)*(HeightsVector/16)) - ...
                       abs(1/H1H2_RATIO)*(HeightsVector/16));
AcceptableHeights = find(Remainders <= HEIGHT_TOLERANCE);

if AcceptableHeights

% This optimization function carries a penalty for both
% increasing the error in the height 2 and changing the
% current height 1.  
  NormDist = abs(PULSE1_HEIGHT - HeightsVector(AcceptableHeights))/32768;
  NormErr = Remainders(AcceptableHeights)/HEIGHT_TOLERANCE;
  PenaltyFunction = ( 1 + ( .5 * ( (NormDist/.1) .^ 2 ) ) ) .* ...
                    ( 1 + NormErr );
  [value,index] = min(PenaltyFunction);
  PULSE1_HEIGHT = HeightsVector(AcceptableHeights(index));
  PULSE2_HEIGHT = 16 * round(PULSE1_HEIGHT/(H1H2_RATIO*16));
else
  FailureFlag = 2;
end

% Set error flag if any of the pulse heights are out of range.
if (PULSE1_HEIGHT < -32768) | (PULSE1_HEIGHT > 32752) | ...
   (PULSE2_HEIGHT < -32768) | (PULSE2_HEIGHT > 32752)
  FailureFlag = 2;
end

% Set error flag if total pulse width is unacceptable
if (PULSE1_DELAY+PULSE1_WIDTH+PULSE2_DELAY+PULSE2_WIDTH) >= OUT_NUM_SAMPLES
  FailureFlag = 1;
end

if FailureFlag
  h = errordlg(['Unable to find an acceptable pre-set waveform. ' ...
               'You might try making the first pulse shorter in ' ...
               'width and/or height, or ' ...
               'increasing WIDTH_TOLERANCE and/or HEIGHT_TOLERANCE.'], ... 
                'TPPreset:','modal');                                        
  figure(h)
  uiwait(h)
  PULSE1_DELAY = d1;
  PULSE2_DELAY = d2;
  PULSE1_HEIGHT = h1;
  PULSE2_HEIGHT = h2;
  PULSE1_WIDTH = w1;
  PULSE2_WIDTH = w2;
  set(findobj('Tag','ShapeLock'),'Value',1)
  TPsw shapelock
end


TXT_UPDATE_FLAGS = 63;
TPNwTxt;
TPPltPls;
